module Hashtbl : sig
  type ('a, 'b) t
  val create : ?path:string -> string -> int -> ('a, 'b) t
  val clear : ('a, 'b) t -> unit
  val add : ('a, 'b) t -> 'a -> 'b -> unit
  val copy : ?path:string -> string -> ('a, 'b) t -> ('a, 'b) t
  val find : ('a, 'b) t -> 'a -> 'b
  val find_all : ('a, 'b) t -> 'a -> 'b list
  val mem : ('a, 'b) t -> 'a -> bool
  val remove : ('a, 'b) t -> 'a -> unit
  val replace : ('a, 'b) t -> 'a -> 'b -> unit
  val iter : ('a -> 'b -> unit) -> ('a, 'b) t -> unit
  val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) t -> 'c -> 'c
  val length : ('a, 'b) t -> int
  (*
  val h_debug : ('a, 'b) t -> unit  (* DEBUG *)
  *)
end

(* BEGIN_SIMEXT *)
module Enum : sig
  type 'a t = 'a list
end
(* END_SIMEXT *)
module ExtHashtbl : sig
  module Hashtbl : sig
    (*
    type ('a, 'b) t = ('a, 'b) Hashtbl.t
    *)
    type ('a, 'b) t

    (* Previous functions *)
    val create : ?path:string -> string -> int -> ('a, 'b) t
    val clear : ('a, 'b) t -> unit
    val add : ('a, 'b) t -> 'a -> 'b -> unit
    val copy : ?path:string -> string -> ('a, 'b) t -> ('a, 'b) t
    val find : ('a, 'b) t -> 'a -> 'b
    val find_all : ('a, 'b) t -> 'a -> 'b list
    val mem : ('a, 'b) t -> 'a -> bool
    val remove : ('a, 'b) t -> 'a -> unit
    val replace : ('a, 'b) t -> 'a -> 'b -> unit
    val iter : ('a -> 'b -> unit) -> ('a, 'b) t -> unit
    val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) t -> 'c -> 'c
    val length : ('a, 'b) t -> int

    (* ExtLib functions *)
    val exists : ('a, 'b) t -> 'a -> bool
    val keys : ('a, 'b) t -> 'a Enum.t
    val values : ('a, 'b) t -> 'b Enum.t
    val enum : ('a, 'b) t -> ('a * 'b) Enum.t
    val of_enum : ('a * 'b) Enum.t -> ?path:string -> string -> ('a, 'b) t
    val find_default : ('a, 'b) t -> 'a -> 'b -> 'b
    val find_option : ('a, 'b) t -> 'a -> 'b option
    val remove_all : ('a, 'b) t -> 'a -> unit
    val map : ('a -> 'b) -> ('c, 'a) t -> ?path:string -> string -> ('c, 'b) t
  end
end

